/*
 * a1-i.l.c
 * Copyright (C) 2011-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <a1-inter.h>
#include <a1-types.h>
#include <e4.h>
#include <fd.h>
#include <lxcall.h>
#include <lxcast.h>

#define POST_BITS \
    X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA | X1f4_LXCALL_POST_LINE

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

typedef struct slip_type {
    const int *args;
    int (*call)(void *, int, void *, struct x1f4_dispatch_type **), lead,
	lock, miss, slip, tail;
    struct a1_instant_type *instant_data;
    struct a1_shuffle_type *shuffle_data;
    struct fxdata_type finite;
    struct x1f4_dispatch_type *dispatch_data;
    struct x1f4_dxcast_type dxcast, serial;
    struct x1f4_function_type function;
    const struct x1f4_function_type *function_text;
    struct x1f4_linetext_type linetext, *linetext_data;
    unsigned class, flags, scale;
    void *back, *clear[3], **track;
} slip_type;

extern const void *const x1f4_a1_walk_link;

static int part_call(void *, int, void *, struct x1f4_dispatch_type **);

static int
part_call(void *back, int status, void *output,
	  struct x1f4_dispatch_type **dispatch)
{
    do {
	int slip;
	struct slip_type *call_data;

	if (status) {
	    break;
	}

	call_data = back;
	status = call_data->call(call_data->back, 0, output, dispatch);
	if (status) {
	    break;
	}

	slip = call_data->slip;
	slip++;
	if (slip != call_data->tail) {
	    int lock;
	    struct a1_instant_type *instant_data;
	    struct a1_shuffle_type *shuffle_data;
	    struct x1f4_dispatch_type *dispatch_data;
	    void **frame, **track;

	    if (slip == call_data->miss) {
		slip = 0;
	    }

	    call_data->slip = slip;

	    instant_data = call_data->instant_data;
	    shuffle_data = call_data->shuffle_data;

	    track = call_data->track;

	    frame = (void *) (call_data + 1);
	    frame++;

	    lock = call_data->lock;

	    _x1f4_a1_keep_call
		(frame, call_data->flags, track[-1],
		 (int *) call_data->function.args, instant_data,
		 call_data->args, call_data->class, lock, call_data->miss,
		 call_data->slip, call_data->scale);

	    /* the args, count, flags set should do */
	    call_data->linetext = *call_data->linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    call_data->clear[2] = call_data->dispatch_data;

	    status = _libx1f4i0_lxcall_rail_call
		(&call_data->linetext, &call_data->function,
		 (void *) x1f4_a1_walk_link, 0, call_data->clear,
		 POST_BITS | X1f4_LXCALL_POST_PLOT, &shuffle_data->p,
		 &shuffle_data->q, &shuffle_data->x,
		 shuffle_data->y.eelookup_data, 1, &lock, &call_data->finite,
		 &call_data->serial);
	    if (status) {
		break;
	    }

	    dispatch_data = call_data->clear[2];

	    if (dispatch_data) {
		call_data->back = dispatch_data->back;
		call_data->call = dispatch_data->call;

		dispatch_data->back = call_data;
		dispatch_data->call = part_call;

		call_data->linetext.function.type = call_data->dxcast.type;

		*dispatch = dispatch_data;
	    }
	} else {
	    *dispatch = NULL;
	}
    } while (0);

    return status;
}


int
_x1f4_a1_lift_call(struct a1_instant_type *instant_data, void *output,
		   void **input, struct x1f4_linetext_type *linetext_data,
		   int head, int tail,
		   const struct x1f4_function_type *function_text, int lead,
		   int lock)
{
    int linked, status;
    const struct x1f4_function_type *function_data;
    unsigned class, count, flags, grade, parts, scale = 0;
    void *beta, **track;

    function_data = &instant_data->fail_node->linetext.function;

    if (function_data->flags & (X1f4_E4_SIDE_LIST | X1f4_E4_SLIP_LIST)) {
	function_data = instant_data->s.function_data;
    }

    flags = function_data->flags;

    count = function_data->count;

    if (head < 0) {
	head += count;
    }
    if (tail < 0) {
	tail += count;
    }

    class = function_text->count - lead;

    grade = class + 1;

    if (lock < 0) {
	lock += grade;
    }

    parts = function_text->flags;

    if ((flags | parts) & X1f4_E4_POST_TYPE) {
	if (linetext_data->function.flags
	    & (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
	    scale = 1;
	} else {
	}
    }

    linked = sizeof(struct slip_type);

    status = instant_data->v.link
	(instant_data->v.data, &beta,
	 linked + (sizeof(void *) + (sizeof(int) << scale)) * grade
	 + sizeof(void *));
    if (status) {
	status = _x1f4_a1_stat_link(instant_data->shuffle_data);
    } else {
	int *args, type;
	struct a1_shuffle_type *shuffle_data;
	struct slip_type *call_data;
	struct x1f4_function_type *function_call;
	void **clear, **frame, *series;

	call_data = beta;

	function_call = &call_data->function;

	function_call->count = grade;

	function_call->flags = scale ? X1f4_E4_POST_TYPE : 0;

	frame = (void *) ((char *) beta + linked);
	frame++;

	function_call->args = args = (void *) (frame + grade);

	track = input[1];

	frame[-1] = track[-1];

	tail++;

	shuffle_data = instant_data->shuffle_data;

	type = linetext_data->function.type;

	series = input[0];

	if (type != X1f4_E4_CASE) {
	    l_USER(&call_data->dxcast, series);

	    call_data->dxcast.call = 0;
	    call_data->dxcast.type = type;

	    series = &call_data->dxcast.data;
	} else {
	}

	_x1f4_a1_hold_call
	    (frame, track, parts, flags, track[-1], args, instant_data,
	     function_text->args, function_data->args, class, lead, lock,
	     count, head, scale);

	clear = call_data->clear;

	clear[0] = series;
	clear[1] = frame;

	call_data->linetext = *linetext_data;

	call_data->linetext.function.type = X1f4_E4_CASE;

	status = _libx1f4i0_lxcall_post_call
	    (&call_data->linetext, function_call, output, 0, clear, POST_BITS,
	     &instant_data->v, &shuffle_data->p, &shuffle_data->q,
	     &shuffle_data->x, shuffle_data->y.eelookup_data);
	if (status) {
	} else {
	    struct x1f4_dispatch_type *dispatch_data;

	    dispatch_data = input[2] = clear[2];

	    if (dispatch_data) {
		call_data->args = function_data->args;
		call_data->back = dispatch_data->back;
		call_data->call = dispatch_data->call;
		call_data->lead = lead;
		call_data->lock = lock;
		call_data->miss = count;
		call_data->slip = head;
		call_data->tail = tail;

		call_data->class = class;
		call_data->flags = flags;
		call_data->scale = scale;
		call_data->track = track;

		call_data->instant_data = instant_data;
		call_data->shuffle_data = shuffle_data;

		call_data->function_text = function_text;
		call_data->linetext_data = linetext_data;

		dispatch_data->back = call_data;
		dispatch_data->call = part_call;

		call_data->linetext.function.type = type;

		call_data->dispatch_data = dispatch_data;
	    }
	}
    }

    return status;
}
