/*
 * a1-j.l.c
 * Copyright (C) 2011-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stddef.h>
#include <string.h>

#include <a1-inter.h>
#include <a1-types.h>
#include <e4.h>
#include <lxcall.h>
#include <lxcast.h>

#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

extern const void *const x1f4_a1_walk_link;

static int beta_call(void *, void *, void **);
static int fine_call(struct a1_instant_type *, void *, void **, void **,
		     struct x1f4_linetext_type *,
		     const struct x1f4_function_type *, unsigned,
		     const struct x1f4_function_type *, unsigned);
static int head_call(void **, unsigned, unsigned, char *, int *,
		     struct a1_instant_type *, const int *,
		     int, unsigned, int, int, unsigned);

static const int c_____c_m[] = {
/* *INDENT-OFF* */
    X1f4_E4_CASE,
    X1f4_E4_MODE
/* *INDENT-ON* */
};
const struct x1f4_function_type _x1f4_a1_j_l = {
/* *INDENT-OFF* */
    "lcall",
    beta_call,
    X1f4_E4_CASE,
    c_____c_m,
    2,
    X1f4_E4_KEEP_CALL | X1f4_E4_LINK_PASS | X1f4_E4_SIDE_LIST
    | X1f4_E4_WALK_LINK,
    5
/* *INDENT-ON* */
};

static int
beta_call(void *context, void *output, void **input)
{
    int status;

    do {
	X1f4_E4_C_MODE news;
	const int *args;
	struct a1_instant_type *instant_data;
	struct x1f4_dxcast_type *dxcast_data;
	const struct x1f4_function_type *function_data, *function_text;
	struct x1f4_linetext_type *linetext_data;
	unsigned class, count, i, scale = 0;
	void **super, **track, **used;

	if (output == x1f4_a1_walk_link) {
	    track = input[1];
	} else {
	    track = input;
	}

	used = track[-1];

	instant_data = used[X1f4_E4_USED_META];

	dxcast_data = I_USER(track[0]);

	if (X1f4_E4_CALL < dxcast_data->type) {
	} else {
	    status = _x1f4_a1_stat_call
		(instant_data->shuffle_data, "lcall", 5);
	    if (1) {
		break;
	    }
	}

	linetext_data = I_USER(&dxcast_data->data);
	if (linetext_data->function.function) {
	} else {
	    status = _x1f4_a1_stat_zero
		(instant_data->shuffle_data, "lcall", 5);
	    if (1) {
		break;
	    }
	}

	function_text = used[X1f4_E4_RENDITION];

	class = function_text->count - 2;

	news = I_MODE(track[1]);
	if (news < -1) {
	    status = _x1f4_a1_stat_news(instant_data->shuffle_data, news);
	    if (1) {
		break;
	    }
	}
	if (class < news * 3) {
	    status = _x1f4_a1_stat_lall
		(instant_data->shuffle_data, news, class + 2);
	    if (1) {
		break;
	    }
	}

	function_data = &instant_data->fail_node->linetext.function;

	if (function_data->flags & (X1f4_E4_SIDE_LIST | X1f4_E4_SLIP_LIST)) {
	    function_data = instant_data->s.function_data;
	}

	class -= news * 3 - 1;

	count = function_data->count;

	args = function_text->args + 2;

	i = news * 3;
	for (; i; i--) {
	    if (*args++ != X1f4_E4_MODE) {
		status = _x1f4_a1_stat_this
		    (instant_data->shuffle_data, *(args - 1),
		     3 * news - i + 2);
		if (1) {
		    break;
		}
	    }
	}
	if (i) {
	    break;
	}

	super = track + 2;

	i = news;
	for (; i; i--) {
	    X1f4_E4_C_MODE lock;

	    lock = I_MODE(*super++);

	    if (lock < -(int) class || !(lock < (int) class)) {
		status = _x1f4_a1_stat_land
		    (instant_data->shuffle_data, lock, class);
		if (1) {
		    break;
		}
	    } else {
		X1f4_E4_C_MODE head, tail;

		head = I_MODE(*super++);
		tail = I_MODE(*super++);

		count = function_data->count;
		if (head < -(int) count || !(head < (int) count)) {
		    status = _x1f4_a1_stat_rend
			(instant_data->shuffle_data, head, count);
		    if (1) {
			break;
		    }
		} else {
		    if (tail < -(int) count || !(tail < (int) count)) {
			status = _x1f4_a1_stat_rend
			    (instant_data->shuffle_data, tail, count);
			if (1) {
			    break;
			}
		    } else {
			if (head < 0) {
			    head += count;
			}
			if (tail < 0) {
			    tail += count;
			}
			if (tail < head) {
			    scale += count - head + tail + 1;
			} else {
			    scale += tail + 1 - head;
			}
		    }
		}
	    }
	}
	if (i) {
	    break;
	}

	status = fine_call
	    (instant_data, output, input, track, linetext_data, function_data,
	     scale, function_text, class - 1);
    } while (0);

    return status;
}


static int
fine_call(struct a1_instant_type *instant_data, void *output, void **input,
	  void **track, struct x1f4_linetext_type *linetext_data,
	  const struct x1f4_function_type *function_data, unsigned scale,
	  const struct x1f4_function_type *function_text, unsigned class)
{
    int status;
    struct x1f4_function_type function;
    unsigned count, flags, parts, *point, shift = 0;
    void *beta;

    flags = function_data->flags;

    count = function_data->count;

    function.count = class + scale;

    parts = function_text->flags;

    function.flags = (flags | parts) & X1f4_E4_POST_TYPE;

    if (function.flags) {
	shift = 1;
    }

    class++;

    point = calloc(2, class * sizeof(unsigned));
    if (point) {
	status = instant_data->v.link
	    (instant_data->v.data, &beta,
	     (sizeof(void *) + (sizeof(int) << shift)) * function.count
	     + sizeof(void *));
	if (status) {
	    status = _x1f4_a1_stat_link(instant_data->shuffle_data);
	} else {
	    X1f4_E4_C_MODE news;
	    const int *args;
	    int *dana;
	    struct a1_shuffle_type *shuffle_data;
	    unsigned i;
	    void *clear[3], **frame, **super;

	    frame = beta;
	    frame++;

	    dana = (void *) (frame + function.count);

	    function.args = dana;

	    frame[-1] = track[-1];

	    news = I_MODE(track[1]);

	    super = track + 2;

	    i = news;
	    for (; i; i--) {
		X1f4_E4_C_MODE head, lock, tail;

		lock = I_MODE(*super++);
		if (lock < 0) {
		    lock += class;
		}

		head = I_MODE(*super++);
		tail = I_MODE(*super++);
		if (head < 0) {
		    head += count;
		}
		if (tail < 0) {
		    tail += count;
		}
		if (tail < head) {
		    point[lock] += count - head + tail + 1;
		} else {
		    point[lock] += tail + 1 - head;
		}
	    }

	    shift = 0;
	    i = class;
	    for (; i; i--) {
		point[class] = shift;
		shift += *point;
		shift++;
		point++;
	    }

	    super = track + 2;

	    i = news;
	    for (; i; i--) {
		X1f4_E4_C_MODE head, lock, tail;

		lock = I_MODE(*super++);
		if (lock < 0) {
		    lock += class;
		}

		head = I_MODE(*super++);
		tail = I_MODE(*super++);
		if (head < 0) {
		    head += count;
		}
		if (tail < 0) {
		    tail += count;
		}

		head_call
		    (frame, parts, flags, track[-1], dana, instant_data,
		     function_data->args, point[lock], count, head, tail,
		     function.count);

		if (tail < head) {
		    point[lock] += count - head + tail + 1;
		} else {
		    point[lock] += tail + 1 - head;
		}
	    }

	    args = function_text->args;
	    args += 2 + news * 3;

	    i = class - 1;
	    if ((flags | parts) & X1f4_E4_POST_TYPE) {
		int *days;
		unsigned night;

		night = function_text->count - 1;
		days = dana + function.count;
		for (; i; i--) {
		    shift = *point++;
		    frame[shift] = *super++;
		    dana[shift] = *args++;
		    days[shift] = args[night];
		}
	    } else {
		for (; i; i--) {
		    shift = *point++;
		    frame[shift] = *super++;
		    dana[shift] = *args++;
		}
	    }

	    point++;
	    point -= class << 1;

	    if (output == x1f4_a1_walk_link) {
		clear[0] = input[0];
		clear[1] = frame;
		clear[2] = input[2];

		frame = clear;
	    }

	    shuffle_data = instant_data->shuffle_data;

	    status = _libx1f4i0_lxcall_post_call
		(linetext_data, &function, output, 0, frame,
		 X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA,
		 &instant_data->v, &shuffle_data->p, &shuffle_data->q,
		 &shuffle_data->x, shuffle_data->y.eelookup_data);
	    if (status) {
	    } else {
		if (output == x1f4_a1_walk_link) {
		    input[2] = clear[2];
		}
	    }
	}

	free(point);
    } else {
	status = _x1f4_a1_stat_link(instant_data->shuffle_data);
    }

    return status;
}


static int
head_call(void **post_data, unsigned parts, unsigned flags,
	  char *used, int *post_args, struct a1_instant_type *instant_data,
	  const int *link_args, int lock,
	  unsigned link, int head, int tail, unsigned lead)
{
    int call;
    struct a1_link_type *link_data;
    struct a1_node_type *node_data;
    struct x1f4_dxcast_type *object_text;
    unsigned class;
    void **rail;

    node_data = instant_data->fail_node;

    link_data = node_data->link_data;

    post_data += lock;

    tail++;

    class = node_data->linetext.function.count;

    object_text = instant_data->s.lead;
    object_text -= class;

    rail = (void *) (object_text + link);
    rail -= class;

    call = head;
    do {
	void *slip;

	if (call == link) {
	    call = 0;
	}

	if (call < class) {
	    slip = used + (integral_q) link_data[call].datatext.link;
	    if (flags & X1f4_E4_POST_TYPE) {
		if (link_args[call + link] & X1f4_E4_POST_XSET) {
		    slip = *(void **) slip;
		}
	    }
	} else {
	    if (flags & X1f4_E4_POST_TYPE) {
		if (link_args[call + link] & X1f4_E4_POST_XSET) {
		    slip = rail[call];
		} else {
		    if (link_args[call] == X1f4_E4_CASE) {
			rail[call] = object_text + call;
			slip = rail + call;
		    } else {
			slip = &object_text[call].data;
		    }
		}
	    } else {
		if (link_args[call] == X1f4_E4_CASE) {
		    rail[call] = object_text + call;
		    slip = rail + call;
		} else {
		    slip = &object_text[call].data;
		}
	    }
	}

	*post_data++ = slip;

	call++;
    } while (call ^ tail);

    post_args += lock;

    if (head < tail) {
	memcpy(post_args, link_args + head, (tail - head) * sizeof(int));
    } else {
	memcpy(post_args, link_args + head, (link - head) * sizeof(int));
	memcpy(post_args + link - head, link_args, tail * sizeof(int));
    }

    if ((flags | parts) & X1f4_E4_POST_TYPE) {
	post_args += lead;

	if (flags & X1f4_E4_POST_TYPE) {
	    link_args += link;

	    if (head < tail) {
		memcpy
		    (post_args, link_args + head, (tail - head) * sizeof(int));
	    } else {
		memcpy
		    (post_args, link_args + head, (link - head) * sizeof(int));
		memcpy(post_args + link - head, link_args, tail * sizeof(int));
	    }
	} else {
	    if (head < tail) {
		memset(post_args, 0, (tail - head) * sizeof(int));
	    } else {
		memset(post_args, 0, (link - head + tail) * sizeof(int));
	    }
	}
    }

    return 0;
}
