/*
 * e4-o.2.c
 * Copyright (C) 2007-2012, 2014, 2015, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>

#include <e4-defs.h>
#include <e4-types.h>

extern const char *_x1f4_e4_type_names[];

static void print_type(FILE *, int);

static void
print_type(FILE *file, int type)
{
    if (type < SUIT) {
	if (type < EVER) {
	    unsigned i, j;

	    j = strlen(_x1f4_e4_type_names[type]);
	    if (j < 12) {
		j = 12 - j;
	    } else {
		j = 0;
	    }
	    i = j >> 1;
	    j -= i;
	    for (; i; i--) {
		fputc(' ', file);
	    }
	    fputs(_x1f4_e4_type_names[type], file);
	    for (; j; j--) {
		fputc(' ', file);
	    }
	} else {
	    fputs("            ", file);
	}
    } else {
	fputs("   <user>   ", file);
    }
}


void
x1f4_print_toperators(FILE *file, const struct e4_odbx_type *const *list,
		      int head)
{
    const struct e4_odbx_type *odbx_data;

    if (head) {
	fputs("    Type    Left Operand      Mid Operand            Right Op"
	      "erand Precedence\n", file);
    }

    odbx_data = *list;
    while (odbx_data) {
	const int *args;
	unsigned i, j;

	if (odbx_data->bits & E2ND_LINK) {
	    const struct e4_2nde_type *e2nd_data;

	    e2nd_data = odbx_data->extension2;
	    if (e2nd_data->bits & RAIL_MARK) {
		const struct e4_odbx_type *odbx_rail;

		print_type(file, odbx_data->type);
		args = odbx_data->args;
		print_type(file, *args);
		j = 6 - odbx_data->size;
		i = j >> 1;
		j -= i;
		for (; i; i--) {
		    fputc(' ', file);
		}
		fputs(odbx_data->name, file);
		for (; j; j--) {
		    fputc(' ', file);
		}
		args++;
		print_type(file, *args);
		fputs("      ", file);
		odbx_rail = e2nd_data->rail;
		args = odbx_rail->args;
		j = 6 - odbx_rail->size;
		i = j >> 1;
		j -= i;
		for (; i; i--) {
		    fputc(' ', file);
		}
		fputs(odbx_rail->name, file);
		for (; j; j--) {
		    fputc(' ', file);
		}
		args++;
		print_type(file, *args);
		fprintf(file, "    %#4o\n", odbx_rail->pass);
	    }
	}

	list++;
	odbx_data = *list;
    }
}
