/*
 * lxline-i.j.c
 * Copyright (C) 2011-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include <l_note.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxline-drive.h>
#include <lxline-inter.h>
#include <lxline-types.h>

#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_TEXT(t)			(*((X1f4_E4_C_TEXT *) (t)))

#define l_TEXT(e, output) \
    {									      \
	X1f4_E4_C_TEXT *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define dxline(list) \
    ((struct dxline_type *) (list))
#define lxline(list) \
    ((struct lxline_type *) (list))

typedef struct rule_type {
    struct fydata_type *fydata_data;
    struct lxline_type *lxline_data;
    unsigned tune;
    void *pass;
} rule_type;

extern const char *const x1f4_c1_empty_string;
extern const void *const x1f4_a1_walk_link;

static int ever_list(struct lxline_type *, struct fydata_type *,
		     struct dxline_type *);
static int rule_line(void *, void *, const char *, unsigned);
static int rule_text(struct fydata_type *, const char *, unsigned,
		     struct rule_type *);

static int
ever_list(struct lxline_type *lxline_data, struct fydata_type *fydata_data,
	  struct dxline_type *dxline_data)
{
    struct rule_type rule;

    rule.fydata_data = fydata_data - 1;
    rule.lxline_data = lxline_data;

    rule.tune = 0;

    return _libx1f4i0_span_sfnote(dxline_data->line, &rule, rule_line);
}


static int
rule_line(void *line, void *data, const char *text, unsigned size)
{
    struct rule_type *rule_data;

    rule_data = line;

    rule_data->fydata_data++;

    return rule_text((void *) rule_data->fydata_data, text, size, rule_data);
}


static int
rule_text(struct fydata_type *fydata_data, const char *text, unsigned size,
	  struct rule_type *rule_data)
{
    int status;

    if (size) {
	unsigned tune;

	size++;

	tune = rule_data->tune;
	if (tune < size || 512 < size) {
	    struct lxline_type *lxline_data;

	    if (512 < size) {
		lxline_data = rule_data->lxline_data;

		status = lxline_data->link_v.link
		    (lxline_data->link_v.data, (void *) fydata_data, size);
		if (status) {
		    status = _libx1f4i0_lxline_stat_link(lxline_data);
		} else {
		    memcpy(I_TEXT(fydata_data), text, size);
		}
	    } else {
		lxline_data = rule_data->lxline_data;

		status = lxline_data->link_v.link
		    (lxline_data->link_v.data, &rule_data->pass, 4096);
		if (status) {
		    status = _libx1f4i0_lxline_stat_link(lxline_data);
		} else {
		    char *pass;

		    pass = rule_data->pass;

		    memcpy(pass, text, size);

		    rule_data->pass = pass + size;
		    rule_data->tune = 4096 - size;

		    l_TEXT(pass, fydata_data);
		}
	    }
	} else {
	    char *pass;

	    status = 0;

	    pass = rule_data->pass;

	    memcpy(pass, text, size);

	    rule_data->pass = pass + size;
	    rule_data->tune = tune - size;

	    l_TEXT(pass, fydata_data);
	}
    } else {
	status = 0;

	l_TEXT((void *) x1f4_c1_empty_string, fydata_data);
    }

    return status;
}


int
_libx1f4i0_lxline_very_list(void *lxline, void *output, void **input,
			    void **track,
			    struct x1f4_linetext_type *linetext_data,
			    unsigned count,
			    const struct x1f4_function_type *function_text,
			    int lead, int jack, void *dxline)
{
    int status;
    void *beta;

    status = lxline(lxline)->link_v.link
	(lxline(lxline)->link_v.data, &beta,
	 count * sizeof(struct fydata_type));
    if (status) {
	status = _libx1f4i0_lxline_stat_link(lxline);
    } else {
	status = ever_list(lxline, beta, dxline);
	if (status) {
	} else {
	    if (output == x1f4_a1_walk_link) {
		status = _libx1f4i0_lxrail_draw_call
		    (lxline, output, input, linetext_data, count,
		     function_text, lead, jack, X1f4_E4_TEXT, beta);
	    } else {
		status = _libx1f4i0_lxrail_jack_call
		    (lxline, output, input, linetext_data, count,
		     function_text, lead, jack, X1f4_E4_TEXT, beta);
	    }
	}
    }

    return status;
}
