/*
 * lxlist-l.d.c
 * Copyright (C) 2008-2011, 2013-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <l_list.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#define I_LIST(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#if __HAVE_PULL_L4LIST__
# define clip(none) \
    ((struct clip_type *) (none))
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
typedef struct clip_type {
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
} clip_type;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int clip_miss(void *, void *);
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int
clip_miss(void *clip, void *data)
{
    int status;
    unsigned link;

    link = effect_link(data);
    if (~link) {
	int (*free)(void *, void *, struct excase_type **,
		    const struct excase_type *);
	struct lxtext_type *lxtext_link;

	lxtext_link =
	    ((struct lxtext_type *) clip(clip)->lxlist_data->link_f.text)
	    + link;

	link_effect(data, ~0);

	free = lxtext_link->lxtype.free;
	if (free) {
	    clip(clip)->status = status = free
		(lxtext_link->lxtype.context, I_USER(byte(data) + 8), NULL,
		 lock_excase(clip(clip)->dxlist_data));
	} else {
	    status = 0;
	}
    } else {
	status = 0;
    }

    return status;
}
#endif				/* __HAVE_PULL_L4LIST__ */


int
_libx1f4i0_lxlist_d_logique(void *context, void *output, void **input)
{
    int status;
    struct dxlist_type *dxlist_data;

    dxlist_data = I_LIST(input[0]);
    if (0) {
    } else {
	X1f4_E4_C_MODE mode;
	unsigned size;

	mode = I_MODE(input[1]);

	size = dxlist_data->size;
	if (mode < -(X1f4_E4_C_MODE) (int) size
	    || !(mode < (X1f4_E4_C_MODE) (int) size)) {
	    status = _libx1f4i0_lxlist_stat_deck(context, size, mode);
	} else {
#if __HAVE_PULL_L4LIST__
            struct clip_type clip;
#else
	    void *data;
#endif				/* __HAVE_PULL_L4LIST__ */

	    if (mode < 0) {
		mode += size;
	    }

#if __HAVE_PULL_L4LIST__
	    clip.dxlist_data = dxlist_data;
	    clip.lxlist_data = context;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
	    status = _libx1f4i0_pull_l4list
		(dxlist_data->list, mode, &clip, clip_miss);
	    if (status) {
		status = _libx1f4i0_lxlist_stat_pull
		    (context, clip.status, status);
	    } else {
		dxlist_data->size = size - 1;
	    }
#else
	    status = _libx1f4i0_peek_l4list(dxlist_data->list, mode, &data);
	    if (status) {
		status = _libx1f4i0_lxlist_stat_list(context, 0, status);
	    } else {
		do {
		    unsigned link;

		    link = effect_link(data);
		    if (~link) {
			int (*free)(void *, void *, struct excase_type **,
				    const struct excase_type *);
			struct lxtext_type *lxtext_link;

			lxtext_link =
			    ((struct lxtext_type *)
			     ((struct lxlist_type *)
			      context)->link_f.text) + link;

			link_effect(data, ~0);

			free = lxtext_link->lxtype.free;
			if (free) {
			    status = free
				(lxtext_link->lxtype.context,
				 I_USER(byte(data) + 8), NULL,
				 lock_excase(dxlist_data));
			    if (status) {
				break;
			    }
			}
		    }

		    status = _libx1f4i0_miss_l4list(dxlist_data->list, mode);
		    if (status) {
			status = _libx1f4i0_lxlist_stat_list
			    (context, 0, status);
		    } else {
			dxlist_data->size = size - 1;
		    }
		} while (0);
	    }
#endif				/* __HAVE_PULL_L4LIST__ */
	}
    }

    return status;
}
