/*
 * lxlist-p.r.c
 * Copyright (C) 2011-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <ct.h>
#include <fd.h>
#include <it.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#define I_LIST(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

typedef struct rule_type {
    int *args;
    void **data;
} rule_type;

extern const void *const x1f4_a1_walk_link;

static int ever_call(void **, void **, unsigned, struct dxlist_type *, int *,
		     const int *, unsigned, int, unsigned, int, int);
static int rule_list(void *, void *);

static int
ever_call(void **post_data, void **line_data, unsigned parts,
	  struct dxlist_type *dxlist_data, int *post_args,
	  const int *line_args, unsigned line, int lock, unsigned link,
	  int head, int tail)
{
    struct rule_type rule;
    unsigned rail;

    rail = line - lock;

    line_data += 5;

    memcpy(post_data, line_data, lock * sizeof(void *));

    post_data += lock;

    line_args += 5;

    memcpy(post_args, line_args, lock * sizeof(int));

    post_args += lock;

    rule.args = post_args;
    rule.data = post_data;

    if (tail < head) {
	_libx1f4i0_over_l4list
	    (dxlist_data->list, head, dxlist_data->size - 1, &rule, rule_list);
	_libx1f4i0_over_l4list
	    (dxlist_data->list, 0, tail, &rule, rule_list);
    } else {
	_libx1f4i0_over_l4list
	    (dxlist_data->list, head, tail, &rule, rule_list);
    }

    memcpy(rule.args, line_args + lock, rail * sizeof(int));

    memcpy(rule.data, line_data + lock, rail * sizeof(void *));

    if (parts & X1f4_E4_POST_TYPE) {
	rule.args += rail;

	line_args += line + 5;

	memcpy(rule.args, line_args, lock * sizeof(int));
	rule.args += lock;

	memset(rule.args, 0, link * sizeof(int));

	memcpy(rule.args + link, line_args + lock, rail * sizeof(int));
    }

    return 0;
}


static int
rule_list(void *line, void *data)
{
    struct rule_type *rule_data;

    rule_data = line;

    *rule_data->args++ = effect_type(data);
    *rule_data->data++ = byte(data) + 8;

    return 0;
}


int
_libx1f4i0_lxlist_r_pending(void *context, void *output, void **input)
{
    int status;

    do {
	X1f4_E4_C_MODE head, lock, tail;
	struct dxlist_type *dxlist_data;
	struct lxlist_type *lxlist_data;
	struct x1f4_dxcast_type *dxcast_data;
	struct x1f4_function_type function;
	const struct x1f4_function_type *function_text;
	struct x1f4_linetext_type *linetext_data;
	unsigned class, count, parts, scale = 0, trip;
	void *beta, **track, **used;

	if (output == x1f4_a1_walk_link) {
	    track = input[1];
	} else {
	    track = input;
	}

	used = track[-1];

	lxlist_data = context;

	dxcast_data = I_USER(track[1]);

	if (X1f4_E4_CALL < dxcast_data->type) {
	} else {
	    status = _libx1f4i0_lxlist_stat_roll(lxlist_data, "l_rcall", 7);
	    break;
	}

	linetext_data = I_USER(&dxcast_data->data);

	if (linetext_data->function.function) {
	} else {
	    status = _libx1f4i0_lxlist_stat_zero(lxlist_data, "l_rcall", 7);
	    break;
	}

	dxlist_data = I_LIST(track[0]);

	count = dxlist_data->size;

	lock = I_MODE(track[2]);

	function_text = used[X1f4_E4_RENDITION];

	class = function_text->count - 5 + 1;

	if (lock < -(int) class || !(lock < (int) class)) {
	    status = _libx1f4i0_lxlist_stat_land(lxlist_data, class, lock);

	    break;
	}

	head = I_MODE(track[3]);

	if (head < -(X1f4_E4_C_MODE) (int) count
	    || !(head < (X1f4_E4_C_MODE) (int) count)) {
	    status = _libx1f4i0_lxlist_stat_deck(lxlist_data, count, head);

	    break;
	}

	tail = I_MODE(track[4]);

	if (tail < -(X1f4_E4_C_MODE) (int) count
	    || !(tail < (X1f4_E4_C_MODE) (int) count)) {
	    status = _libx1f4i0_lxlist_stat_deck(lxlist_data, count, tail);

	    break;
	}

	if (lock < (X1f4_E4_C_MODE) 0) {
	    lock += class;
	}

	class--;

	if (head < (X1f4_E4_C_MODE) 0) {
	    head += count;
	}
	if (tail < (X1f4_E4_C_MODE) 0) {
	    tail += count;
	}

	if (tail < head) {
	    count = count - head + tail + 1;
	} else {
	    count = tail + 1 - head;
	}

	function.count = count + class;

	parts = function_text->flags;

	function.flags = parts & X1f4_E4_POST_TYPE;

	if (function.flags) {
	    scale = 1;
	}

	trip = (sizeof(void *) + (sizeof(int) << scale)) * function.count
	    + sizeof(void *);

	status = lxlist_data->link_v.link
	    (lxlist_data->link_v.data, &beta, trip);
	if (status) {
	    status = _libx1f4i0_lxlist_stat_link(lxlist_data);
	} else {
	    void *clear[3], **frame;

	    frame = beta;
	    frame++;

	    function.args = (void *) (frame + function.count);

	    frame[-1] = used;

	    ever_call
		(frame, track, parts, dxlist_data, (int *) function.args,
		 function_text->args, class, lock, count, head, tail);

	    if (output == x1f4_a1_walk_link) {
		clear[0] = input[0];
		clear[1] = frame;
		clear[2] = input[2];

		frame = clear;
	    }

	    status = _libx1f4i0_lxcall_post_call
		(linetext_data, &function, output, 0, frame,
		 X1f4_LXCALL_POST_ARGS | X1f4_LXCALL_POST_DATA,
		 &lxlist_data->link_v, &lxlist_data->link_p,
		 &lxlist_data->link_q, &lxlist_data->link_e,
		 lxlist_data->trap_e.eelookup_data);
	    if (status) {
	    } else {
		if (output == x1f4_a1_walk_link) {
		    input[2] = clear[2];
		}
	    }
	}
    } while (0);

    return status;
}
