/*
 * lxlist-t.g.c
 * Copyright (C) 2008-2011, 2013-2015, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include <e4.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#define I_LIST(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_TEXT(t)			(*((X1f4_E4_C_TEXT *) (t)))

#define l_TEXT(e, output) \
    {									      \
	X1f4_E4_C_TEXT *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

int
_libx1f4i0_lxlist_g_class_t(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
    struct lxtext_type *lxtext_data;
    unsigned size;

    lxtext_data = context;

    dxlist_data = I_LIST(input[1]);

    lxlist_data = lxtext_data->text;

    mode = I_MODE(input[2]);

    size = dxlist_data->size;
    if (mode < -(X1f4_E4_C_MODE) (int) size
	|| !(mode < (X1f4_E4_C_MODE) (int) size)) {
	status = _libx1f4i0_lxlist_stat_deck(lxlist_data, size, mode);
    } else {
	void *data;

	if (mode < 0) {
	    mode += size;
	}

	status = _libx1f4i0_peek_l4list(dxlist_data->list, mode, &data);
	if (status) {
	    status = _libx1f4i0_lxlist_stat_list(lxlist_data, 0, status);
	} else {
	    unsigned type;

	    type = effect_type(data);
	    if (type != X1f4_E4_TEXT) {
		status = _libx1f4i0_lxlist_stat_type
		    (lxlist_data, mode, type, lxtext_data->lxtype.code);
	    } else {
		X1f4_E4_C_TEXT post;

		post = I_TEXT(byte(data) + 8);

		status = _libx1f4i0_lxcall_swap_text
		    (&lxlist_data->ever_a, *input, post, strlen(post) + 1);
		if (status) {
		} else {
		    l_TEXT(I_TEXT(*input), output);
		}
	    }
	}
    }

    return status;
}
