/*
 * lxlist-t.x.c
 * Copyright (C) 2008-2011, 2013-2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxlist-config.h>

#include <e4.h>
#include <exerrors.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

#define I_LIST(t)			(*((X1f4_E4_C_USER *) (t)))
#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_TEXT(t)			(*((X1f4_E4_C_TEXT *) (t)))

#define l_TEXT(e, output) \
    {									      \
	X1f4_E4_C_TEXT *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#if __HAVE_PULL_L4LIST__
# define clip(none) \
    ((struct clip_type *) (none))
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
typedef struct clip_type {
    X1f4_E4_C_MODE mode;
    int status;
    struct lxlist_type *lxlist_data;
    void *output;
} clip_type;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int clip_miss(void *, void *);
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
static int
clip_miss(void *clip, void *data)
{
    int status;
    unsigned type;

    type = effect_type(data);
    if (type != X1f4_E4_TEXT) {
	clip(clip)->status = _libx1f4i0_lxlist_stat_type
	    (clip(clip)->lxlist_data, clip(clip)->mode, type, X1f4_E4_TEXT);

	status = 1;
    } else {
	status = _libx1f4i0_lxcall_pick_text
	    (&clip(clip)->lxlist_data->ever_a, byte(data) + 8);
	if (status) {
	    clip(clip)->status = status;
	} else {
	    l_TEXT(I_TEXT(byte(data) + 8), clip(clip)->output);
	}
    }

    return status;
}
#endif				/* __HAVE_PULL_L4LIST__ */


int
_libx1f4i0_lxlist_x_class_t(void *context, void *output, void **input)
{
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
    struct lxtext_type *lxtext_data;

    lxtext_data = context;

    lxlist_data = lxtext_data->text;

    dxlist_data = I_LIST(input[0]);
    if (0) {
    } else {
	X1f4_E4_C_MODE mode;
	unsigned size;

	mode = I_MODE(input[1]);

	size = dxlist_data->size;
	if (mode < -(X1f4_E4_C_MODE) (int) size
	    || !(mode < (X1f4_E4_C_MODE) (int) size)) {
	    status = _libx1f4i0_lxlist_stat_deck(lxlist_data, size, mode);
	} else {
#if __HAVE_PULL_L4LIST__
	    struct clip_type clip;
#else
	    void *data, *list;
#endif				/* __HAVE_PULL_L4LIST__ */

	    if (mode < 0) {
		mode += size;
	    }

#if __HAVE_PULL_L4LIST__
	    clip.lxlist_data = lxlist_data;
	    clip.mode = mode;
	    clip.output = output;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
#else
	    list = dxlist_data->list;
#endif				/* __HAVE_PULL_L4LIST__ */

#if __HAVE_PULL_L4LIST__
	    status = _libx1f4i0_pull_l4list
		(dxlist_data->list, mode, &clip, clip_miss);
	    if (status) {
		if (status == X1f4_L4LIST_CALL_ERROR) {
		    status = clip.status;
		} else {
		    status = _libx1f4i0_lxlist_stat_list
			(lxlist_data, mode, status);
		}
	    } else {
		dxlist_data->size = size - 1;
	    }
#else
	    status = _libx1f4i0_peek_l4list(list, mode, &data);
	    if (status) {
		status = _libx1f4i0_lxlist_stat_list(lxlist_data, 0, status);
	    } else {
		unsigned type;

		type = effect_type(data);
		if (type != X1f4_E4_TEXT) {
		    status = _libx1f4i0_lxlist_stat_type
			(lxlist_data, mode, type, X1f4_E4_TEXT);
		} else {
		    status = _libx1f4i0_lxcall_pick_text
			(&lxlist_data->ever_a, byte(data) + 8);
		    if (status) {
		    } else {
			l_TEXT(I_TEXT(byte(data) + 8), output);

			link_effect(data, ~0);

			status = _libx1f4i0_miss_l4list(list, mode);
			if (status) {
			    status = _libx1f4i0_lxlist_stat_list
				(lxtext_data->text, 0, status);
			} else {
			    dxlist_data->size = size - 1;
			}
		    }
		}
	    }
#endif				/* __HAVE_PULL_L4LIST__ */
	}
    }

    return status;
}
