/*
 * lxnorm-a.x.c
 * Copyright (C) 2008-2011, 2013, 2014, 2016, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <dt.h>
#include <e4.h>
#include <exerrors.h>
#include <frame.h>
#include <lxcall.h>
#include <lxnorm.h>
#include <nl.h>
#include <tcvset.h>

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (output);							      \
	*l = (e);							      \
    }

#define lxnorm(list) \
    ((struct lxnorm_type *) (list))

#define tccase(vset) \
    ((struct x1f4_tccase_type *) (vset))

typedef struct upcast_type {
    X1f4_E4_C_USER user;
    int (*slip)(void *, void **);
    void *text;
} upcast_type;

static int slip_node(void *);

static int
slip_node(void *news)
{
    int (*slip)(void *, void **), status;
    struct upcast_type *upcast_data;

    upcast_data = news;

    slip = upcast_data->slip;
    if (slip) {
        status = slip(upcast_data->text, &upcast_data->user);
    } else {
        status = 0;
    }

    return status;
}


int
x1f4_push_lxnorm(void *lxnorm, void **copy, void *line,
		 const struct x1f4_nodelink_type *nodelink_data)
{
    int status;

    do {
	struct upcast_type *upcast_data;
	void *flat;

	flat = *copy;
	if (flat) {
	    struct x1f4_frame_type *frame_data;
	    void **mind, *news;

	    frame_data = &lxnorm(lxnorm)->nice_1.east_1.diff_v;

	    status = frame_data->pick
		(frame_data->data, &mind, slip_node,
		 sizeof(struct upcast_type), &news);
	    if (status) {
		status = _libx1f4i0_lxcall_stat_link
		    (&lxnorm(lxnorm)->nice_1.east_1.diff_e);
		if (1) {
		    break;
		}
	    } else {
		*mind = news;

		upcast_data = news;
	    }
	} else {
	    upcast_data = NULL;
	}

	status = tccase(lock_lnline(line)->text)->miss
	    (&lock_lnline(line)->pset, &lock_lnline(line)->text, nodelink_data,
	     (void *) &((struct lxnorm_type *) lxnorm)->nice_1.east_1.diff_m);
	if (status) {
	    status = X1f4_EX_CRITICAL;

	    if (upcast_data) {
		upcast_data->slip = NULL;
	    }

	    break;
	} else {
	    if (upcast_data) {
		struct x1f4_datatype_type *datatype_data;

		datatype_data = lxnorm(lxnorm)->nice_t.datatype;

		upcast_data->slip = datatype_data->slip;
		upcast_data->text = datatype_data->context;

		upcast_data->user = flat;
	    }
	}

	*copy = (void *) line;
    } while (0);

    return status;
}
