/*
 * lxrail.h
 * Copyright (C) 2016, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxrail_H__
#define __LIBx1f4_lxrail_H__

#include <ct.h>
#include <e4.h>
#include <fd.h>
#include <it.h>
#include <lxplus.h>
#include <nt.h>
#include <progress.h>

#define _libx1f4i0_lxrail_cfix_rail	x1f4_cfix_lxrail
#define _libx1f4i0_lxrail_fold_rail	x1f4_fold_lxrail

#undef ever_a

#define ever_a				ever_r.over_1.rule_1.nice_1.east_1

#undef link_e
#undef link_m
#undef link_v
#undef link_w

#define link_e \
    ever_r.over_1.rule_1.nice_1.east_1.diff_e
#define link_m \
    ever_r.over_1.rule_1.nice_1.east_1.diff_m
#define link_v \
    ever_r.over_1.rule_1.nice_1.east_1.diff_v
#define link_w \
    ever_r.over_1.rule_1.nice_1.east_1.diff_w

#undef trap_e

#define trap_e				ever_r.over_1.rule_1.nice_1.east_e

#undef link_t

#define link_t				ever_r.over_1.rule_1.nice_t

#undef link_r
#undef link_s

#define link_r				ever_r.over_1.rule_r
#define link_s				ever_r.over_1.rule_s

#define link_f				ever_r.over_f
#define link_p				ever_r.over_p
#define link_q				ever_r.over_q

#define lxtype_type                     x1f4_nodetype_type

typedef struct affect_type {
    int link, type;
    struct fxdata_type fxdata, fxtext;
} affect_type;

typedef struct depart_type {
    struct x1f4_fpdetail_type *fpdetail_qlaw, *fpdetail_xlaw;
    struct x1f4_ftaa_type *ftaa_qlaw, *ftaa_xlaw;
    struct x1f4_ftca_type *ftca_qlaw, *ftca_xlaw;
    const void *screen_data;
} depart_type;

typedef struct fydata_type {
    union {
	X1f4_E4_C_MODE mode;
	X1f4_E4_C_TEXT text;
    } data;
} fydata_type;

typedef struct lxmiss_type {
    union {
	unsigned miss;
	void *data;
    } data;
} lxmiss_type;

typedef struct lxtext_type {
    struct lxtype_type lxtype;
    void *text;
} lxtext_type;

typedef struct affine_type {
    struct x1f4_nodelink_type *nodelink_data;
    struct lxtext_type *lxtext_data;
    unsigned miss, size;
} affine_type;

typedef struct lxrail_type {
    struct lxplus_type over_1;
    struct {
	void *data, *side, *text;
    } over_f;
    struct x1f4_iturn_type over_p;
    struct x1f4_cturn_type over_q;
} lxrail_type;

extern int _libx1f4i0_lxrail_cfix_rail(void *, unsigned, void *);
extern int _libx1f4i0_lxrail_draw_call
    (void *, void *, void **, struct x1f4_linetext_type *, unsigned,
     const struct x1f4_function_type *, int, int, int, void *);
extern int _libx1f4i0_lxrail_fold_rail(void *, unsigned *);
extern int _libx1f4i0_lxrail_form_call
    (void *, void *, void **, struct x1f4_linetext_type *, unsigned,
     const struct x1f4_function_type *, int, int, int, int, void *);
extern int _libx1f4i0_lxrail_hold_call
    (void **, void **, unsigned, int *, const int *, unsigned, int, int);
extern int _libx1f4i0_lxrail_jack_call
    (void *, void *, void **, struct x1f4_linetext_type *, unsigned,
     const struct x1f4_function_type *, int, int, int, void *);
extern int _libx1f4i0_lxrail_keep_call
    (void **, void **, unsigned, int *, const int *, unsigned, int, int, int);
extern int _libx1f4i0_lxrail_line_past
    (struct depart_type *, unsigned, void **);
extern int _libx1f4i0_lxrail_link_past
    (struct x1f4_linetrap_type *, struct depart_type *, const char *, unsigned,
     const char *, unsigned);
extern int _libx1f4i0_lxrail_loop_call
    (void *, void *, void **, struct x1f4_linetext_type *, unsigned,
     const struct x1f4_function_type *, int, int, int, int, void *);
extern int _libx1f4i0_lxrail_pass_type
    (void *, struct x1f4_progress_type **, char **,
     const struct x1f4_progress_type *, const struct x1f4_progress_type *,
     const char *, unsigned, const char *, unsigned);
extern int _libx1f4i0_lxrail_post_data
    (void *, void *, unsigned, unsigned, const void *);
extern int _libx1f4i0_lxrail_slip_roll(void *);

#endif				/* __LIBx1f4_lxrail_H__ */
